/*
 * deviced
 *
 * Copyright (c) 2014-2016 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <assert.h>
#include <stdbool.h>
#include <hw/ir.h>
#include <libgdbus/dbus-system.h>
#include "core/devices.h"
#include "core/common.h"
#include "core/log.h"

static struct ir_device *ir_dev;

static GVariant *dbus_ir_is_available(GDBusConnection *conn,
	const gchar *sender, const gchar *path, const gchar *iface, const gchar *name,
	GVariant *param, GDBusMethodInvocation *invocation, gpointer user_data)
{
	int ret = 0;
	bool val;

	if (!ir_dev) {
		ret = -ENODEV;
		goto exit;
	}

	if (!ir_dev->is_available) {
		ret = -ENODEV;
		goto exit;
	}

	ret = ir_dev->is_available(&val);
	if (ret >= 0)
		ret = val;

exit:
	return g_variant_new("(i)", ret);
}

static GVariant *dbus_ir_transmit(GDBusConnection *conn,
	const gchar *sender, const gchar *path, const gchar *iface, const gchar *name,
	GVariant *param, GDBusMethodInvocation *invocation, gpointer user_data)
{
	int ret = 0;
	int size;
	int i = 0;
	int *freq_pattern = NULL;
	GVariantIter *iter = NULL;

	if (!ir_dev) {
		ret = -ENODEV;
		goto exit;
	}

	if (!ir_dev->transmit) {
		ret = -ENODEV;
		goto exit;
	}

	g_variant_get(param, "(ai)", &iter);

	size = g_variant_iter_n_children(iter);
	freq_pattern = (int*)malloc(sizeof(int) * size);

	while (g_variant_iter_next(iter, "i", &freq_pattern[i++]))
		;

	g_variant_iter_free(iter);

	_I("frequency : %d, pattern_size: %d", freq_pattern[0], size);

	ret = ir_dev->transmit(freq_pattern, size);

exit:
	free(freq_pattern);
	return g_variant_new("(i)", ret);
}

static GVariant *dbus_set_ir_command(GDBusConnection *conn,
	const gchar *sender, const gchar *path, const gchar *iface, const gchar *name,
	GVariant *param, GDBusMethodInvocation *invocation, gpointer user_data)
{
	int ret = -ENOTSUP;

	_E("The method is not supported. Use device_ir_transmit() of capi-system-device");

	return g_variant_new("(i)", ret);
}

static const dbus_method_s dbus_methods[] = {
	{ "IRIsAvailable", NULL, "i", dbus_ir_is_available},
	{ "TransmitIR", "ai", "i", dbus_ir_transmit},
	/* Add methods here */
};

static const dbus_interface_u dbus_interface = {
	.oh = NULL,
	.name = DEVICED_INTERFACE_IR,
	.methods = dbus_methods,
	.nr_methods = ARRAY_SIZE(dbus_methods),
};

static const dbus_method_s dbus_methods_legacy[] = {
	{ "SetIrCommand",      "s",   "i", dbus_set_ir_command },
	/* Add methods here */
};

static const dbus_interface_u dbus_interface_legacy = {
	.oh = NULL,
	.name = DEVICED_INTERFACE_LED,
	.methods = dbus_methods_legacy,
	.nr_methods = ARRAY_SIZE(dbus_methods_legacy),
};


static int ir_probe(void *data)
{
	struct hw_info *info;
	int ret;

	if (ir_dev)
		return 0;

	ret = hw_get_info(IR_HARDWARE_DEVICE_ID,
			(const struct hw_info **)&info);

	if (ret < 0) {
		_E("Fail to load ir(%d)", ret);
		return -ENODEV;
	}

	if (!info->open) {
		_E("Failed to open ir device; open(NULL)");
		return -ENODEV;
	}

	ret = info->open(info, NULL, (struct hw_common**)&ir_dev);
	if (ret < 0) {
		_E("Failed to get ir device structure (%d)", ret);
		return ret;
	}

	_I("ir device structure load success");
	return 0;
}

static void ir_init(void *data)
{
	int ret;

	ret = dbus_handle_add_dbus_object(NULL, DEVICED_PATH_IR, &dbus_interface);
	if (ret < 0)
		_E("fail to init dbus method(%d)", ret);

	ret = dbus_handle_add_dbus_object(NULL, DEVICED_PATH_LED, &dbus_interface_legacy);
	if (ret < 0)
		_E("fail to init dbus method(%d)", ret);

}

static void ir_exit(void *data)
{
	struct hw_info *info;

	if (!ir_dev)
		return;

	info = ir_dev->common.info;
	if (!info)
		free(ir_dev);
	else
		info->close((struct hw_common *)ir_dev);

	ir_dev = NULL;
}

static const struct device_ops ir_device_ops = {
	.name     = "ir",
	.probe    = ir_probe,
	.init     = ir_init,
	.exit     = ir_exit,
};

DEVICE_OPS_REGISTER(&ir_device_ops)

